package com.gic.haoban.manage.service.service.out.impl;

import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.ToolUtil;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.dto.StoreGroupDTO;
import com.gic.enterprise.api.service.StoreGroupService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.Constant;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.manage.api.dto.DealParamMqDTO;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.SyncSingleDealDTO;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.DealSyncOperationApiService;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.service.entity.TabHaobanDepartment;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.entity.TabHaobanSyncTask;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import com.task.allocation.api.AbstractTaskAllocationNewOperation;
import com.task.allocation.exception.TaskAllocationException;
import com.task.allocation.qo.TaskAllocationComputed;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * Created 2020/4/9.
 *
 * @author hua
 */
@Service
public class DealSyncOperationApiServiceImpl extends AbstractTaskAllocationNewOperation implements DealSyncOperationApiService {

    private static final Logger logger = LoggerFactory.getLogger(DealSyncOperationApiServiceImpl.class);

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private StoreGroupService storeGroupService;

    @Autowired
    private PreDealService preDealService;

    @Autowired
    private DepartmentApiService departmentApiService;

    @Autowired
    private SyncTaskService syncTaskService;

    @Autowired
    private StoreService storeService;

    @Override
    public void departmentDealMq(String params) {
        logger.info("mq处理中:{}", params);
        DealParamMqDTO dealParamMqDTO = JSONObject.parseObject(params, DealParamMqDTO.class);
        TabHaobanPreDealLog dataPre = preDealService.getByDataId(dealParamMqDTO.getTaskId()
                , dealParamMqDTO.getData(), PreDealStatusEnum.all.getVal());

        if (dataPre == null) {
            logger.info("待处理数据不存在:{}", params);
            return;
        }
        //部门处理
        if (dataPre.getDataType().equals(PreDealTypeEnum.group.getVal())) {
            DepartmentDTO departmentDTO = departmentApiService.selectByRelatedId(dataPre.getDataId());
            if (departmentDTO != null) {
                logger.info("已经同步过了:{}", dataPre.getDataId());
                dealExceptionDepartmenr(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "该部门不存在");
                return;
            }

            TabHaobanDepartment pParentDepartment = departmentService.selectByRelatedId(dataPre.getpDataId());
            if (null == pParentDepartment) {
                logger.info("不存在该父部门：c:{},p:{}", dataPre.getDataId(), dataPre.getpDataId());
                return;
            }
            StoreGroupDTO groupDTO = storeGroupService.getStoreGroupById(dealParamMqDTO.getData());
            if (null == groupDTO) {
                logger.info("该部门不存在");
                dealExceptionDepartmenr(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "该部门不存在");
                return;
            }
            DepartmentDTO department = new DepartmentDTO();
            department.setParentDepartmentId(pParentDepartment.getDepartmentId());
            department.setWxEnterpriseId(pParentDepartment.getWxEnterpriseId());
            department.setDepartmentName(groupDTO.getStoreGroupName());
            department.setChainId(pParentDepartment.getChainId() + Constant.ID_SEPARATOR + dealParamMqDTO.getData());
            department.setChainName(pParentDepartment.getChainName() + Constant.NAME_SEPARATOR + department.getDepartmentName());
            department.setIsStore(0);
            department.setLevel(pParentDepartment.getLevel() + 1);
            department.setRelatedId(dealParamMqDTO.getData());
            department.setSort(groupDTO.getStoreGroupSort());
            //状态 处理状态
            boolean resultFlag = true;

            try {
                ServiceResponse add = departmentApiService.add(department);
                if (add.getCode() != 1) {
                    resultFlag = false;
                    logger.info("处理失败");
                    return;
                }
            } catch (Exception e) {
                resultFlag = false;
                e.printStackTrace();
                logger.info("处理异常");
            } finally {
                if (!resultFlag) {
                    dealExceptionDepartmenr(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "处理异常或失败");
                } else {
                    dealSuccessDepartmenr(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getWxEnterpriseId());

                }
            }
            //门店处理
        } else if (dataPre.getDataType().equals(PreDealTypeEnum.group.getVal())) {
            logger.info("门店处理：{}", params);
            storeDealSingle(dealParamMqDTO, dataPre);

            //成员处理
        } else if (dataPre.getDataType().equals(PreDealTypeEnum.group.getVal())) {
            logger.info("店员处理：{}", params);

        } else {
            logger.info("不是正常数据");
        }

    }

    /**
     * 门店处理
     *
     * @param dealParamMqDTO
     * @param dataPre
     */
    private void storeDealSingle(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
        DepartmentDTO departmentDTO = departmentApiService.selectByRelatedId(dataPre.getDataId());
        if (departmentDTO != null) {
            logger.info("已经同步过了:{}", dataPre.getDataId());
            dealExceptionDepartmenr(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "该部门不存在");
            return;
        }

        TabHaobanDepartment pParentDepartment = departmentService.selectByRelatedId(dataPre.getpDataId());
        if (null == pParentDepartment) {
            logger.info("不存在该父部门：c:{},p:{}", dataPre.getDataId(), dataPre.getpDataId());
            return;
        }
        StoreDTO store = storeService.getStore(dealParamMqDTO.getData());
        if (null == store) {
            logger.info("该门店不存在");
            dealExceptionDepartmenr(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "该部门不存在");
            return;
        }
        DepartmentDTO department = new DepartmentDTO();
        department.setParentDepartmentId(pParentDepartment.getDepartmentId());
        department.setWxEnterpriseId(pParentDepartment.getWxEnterpriseId());
        department.setDepartmentName(store.getStoreName());
        department.setChainId(pParentDepartment.getChainId() + Constant.ID_SEPARATOR + dealParamMqDTO.getData());
        department.setChainName(pParentDepartment.getChainName() + Constant.NAME_SEPARATOR + department.getDepartmentName());
        department.setIsStore(1);
        department.setLevel(pParentDepartment.getLevel() + 1);
        department.setRelatedId(dealParamMqDTO.getData());
        //状态 处理状态
        boolean resultFlag = true;

        try {
            ServiceResponse add = departmentApiService.add(department);
            if (add.getCode() != 1) {
                resultFlag = false;
                logger.info("处理失败");
                return;
            }
        } catch (Exception e) {
            resultFlag = false;
            e.printStackTrace();
            logger.info("处理异常");
        } finally {
            if (!resultFlag) {
                dealExceptionDepartmenr(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "处理异常或失败");
            } else {
                dealSuccessDepartmenr(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getWxEnterpriseId());

            }
        }
    }

    /**
     * 校验部门同步任务成功与否
     *
     * @param taskId
     */
    private void checkDepartmentTask(String taskId) {
        boolean b = preDealService.checkTask(taskId, PreDealTypeEnum.group.getVal());
        if (!b) {
            return;
        }
        logger.info("部门继续成功，开始门店同步:{}", taskId);
        String key = "haoban_sync_department_task_" + taskId;
        RedisUtil.lock(key, 3L);
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        //同步成功 进入门店处理
        if (syncTask.getStatusFlag().equals(SyncTaskStatusEnum.init.getVal())) {
            syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.store_sync.getVal());
            List<TabHaobanPreDealLog> dealLogs = preDealService.listByTaskId(taskId, PreDealTypeEnum.store.getVal(), PreDealStatusEnum.pre.getVal());
            if (CollectionUtils.isEmpty(dealLogs)) {
                syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.compute.getVal());
            } else {
                logger.info("处理门店开始");
                Set<String> dataIds = dealLogs.stream().map(TabHaobanPreDealLog::getDataId).collect(Collectors.toSet());
                dealDepartmentToMq(taskId, dataIds, SyncTaskStatusEnum.store_sync);
            }
        }
        RedisUtil.unlock(key);

    }

    /**
     * 部门异常处理
     *
     * @param dataId
     * @param reason
     */
    private void dealExceptionDepartmenr(String taskId, String dataId, String enterpriseId, String reason) {
        List<String> ids = storeGroupService.getStoreGroupIdsByParentId(enterpriseId, dataId);
        if (CollectionUtils.isEmpty(ids)) {
            logger.info("下面层级不存在");
            //todo 是否需要加锁判断
            return;
        }
        //todo 需要优化 测试
        for (String id : ids) {
            preDealService.updateStatusByDataId(taskId, id, PreDealStatusEnum.exception.getVal());
        }
        checkDepartmentTask(taskId);
    }

    /**
     * 部门成功处理
     *
     * @param dataId
     */
    private void dealSuccessDepartmenr(String taskId, String dataId, String wxEnterpriseId) {
        boolean b = preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.computed.getVal());
        if (!b) {
            logger.info("成功修改错误！{}", dataId);
            return;
        }
        List<StoreDTO> storeDTOS = storeService.listStoreByStoreGroupId(dataId);
        //预处理门店 写入预处理表
        if (CollectionUtils.isNotEmpty(storeDTOS)) {
            List<TabHaobanPreDealLog> preDealLogList = storeDTOS.stream().map(storeDTO -> {
                TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
                dealLog.setDataId(storeDTO.getStoreId());
                dealLog.setpDataId(dataId);
                dealLog.setDataType(PreDealTypeEnum.store.getVal());
                dealLog.setStatusFlag(0);
                dealLog.setTaskId(taskId);
                dealLog.setEnterpriseId(storeDTO.getEnterpriseId());
                dealLog.setWxEnterpriseId(wxEnterpriseId);
                return dealLog;
            }).collect(Collectors.toList());
            preDealService.insert(preDealLogList);
        }
        List<TabHaobanPreDealLog> list = preDealService.listByPDataId(taskId, dataId, PreDealStatusEnum.pre.getVal());
        if (CollectionUtils.isNotEmpty(list)) {
            Set<String> mid = list.stream().map(tab -> tab.getDataId()).collect(Collectors.toSet());
            dealDepartmentToMq(taskId, mid, SyncTaskStatusEnum.group_sync);
        } else {
            checkDepartmentTask(taskId);
        }
    }

    @Override
    public void dealDepartment(String taskId, String wxEnterpriseId
            , List<SyncSingleDealDTO> addList, List<SyncSingleDealDTO> editList) {

        logger.info("初始开始:{}", taskId);
        //处理修改
        dealEidtSync(editList);

        if (CollectionUtils.isEmpty(addList)) {
            return;
        }
        logger.info("处理分组信息开始：{}", taskId);
        dealAddSync(taskId, wxEnterpriseId, editList);

        //加入队列处理
        logger.info("初始化到队列：{}", taskId);
        dealDepartmentInit(taskId, wxEnterpriseId, addList);

    }

    @Override
    public String createTask(String taskName, String userId, String desc) {
        TabHaobanSyncTask tabHaobanSyncTask = new TabHaobanSyncTask();
        tabHaobanSyncTask.setAddUser(userId);
        tabHaobanSyncTask.setTaskName(taskName);
        tabHaobanSyncTask.setTaskDesc(desc);
        tabHaobanSyncTask.setStatusFlag(SyncTaskStatusEnum.init.getVal());
        tabHaobanSyncTask.setTaskId(ToolUtil.randomUUID());
        syncTaskService.createTask(tabHaobanSyncTask);
        return tabHaobanSyncTask.getTaskId();

    }

    /**
     * 处理修改数据
     *
     * @param editList
     */
    private void dealEidtSync(List<SyncSingleDealDTO> editList) {
        if (CollectionUtils.isEmpty(editList)) {
            return;
        }
        //先处理修改的
        editList.forEach(syncSingleDealDTO -> {
            // TODO: 2020/4/9  需要优化
            TabHaobanDepartment department = departmentService.selectById(syncSingleDealDTO.getDepartmentId());
            department.setParentDepartmentId(syncSingleDealDTO.getParentDepartmentId());
            departmentApiService.edit(EntityUtil.changeEntityByJSON(DepartmentDTO.class, department));
        });
    }

    /**
     * 执行新增同步
     *
     * @param taskId
     * @param wxEnterpriseId
     * @param addList
     */
    private void dealAddSync(String taskId, String wxEnterpriseId, List<SyncSingleDealDTO> addList) {
        //获取企业与分组map
        Map<String, List<String>> eidListMap = addList.stream().filter(dto -> {
            String departmentId = dto.getDepartmentId();
            TabHaobanDepartment department = departmentService.selectById(departmentId);
            if (department != null) {
                return false;
            }
            Integer storeFlag = dto.getStoreFlag();
            if (storeFlag.equals(1)) {
                return false;
            }
            return true;
        }).collect(Collectors.groupingBy(SyncSingleDealDTO::getEnterpriseId
                , Collectors.mapping(SyncSingleDealDTO::getRelatedId, Collectors.toList())));

        //放入预处理表
        eidListMap.forEach((k, v) -> {
            //获取所有分组列表
            List<StoreGroupDTO> groupDTOList = storeGroupService.listStoreGroupAndChildren(v.toArray(new String[]{}), k);
            List<TabHaobanPreDealLog> dealList = groupDTOList.stream().map(storeGroupDTO -> {
                TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
                dealLog.setDataId(storeGroupDTO.getStoreGroupId());
                dealLog.setpDataId(storeGroupDTO.getParentGroupId());
                dealLog.setDataType(PreDealTypeEnum.group.getVal());
                dealLog.setStatusFlag(0);
                dealLog.setTaskId(taskId);
                dealLog.setEnterpriseId(storeGroupDTO.getEnterpriseId());
                dealLog.setWxEnterpriseId(wxEnterpriseId);
                return dealLog;
            }).collect(Collectors.toList());

            preDealService.insert(dealList);
        });

        //处理门店信息
        List<TabHaobanPreDealLog> storeList = addList.stream().filter(dto -> {
            return dto.getStoreFlag() == 1;
        }).map(dto -> {
            TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
            dealLog.setDataId(dto.getRelatedId());
            dealLog.setpDataId(dto.getParentDepartmentId());
            dealLog.setDataType(1);
            dealLog.setStatusFlag(0);
            dealLog.setTaskId(taskId);
            dealLog.setWxEnterpriseId(wxEnterpriseId);
            return dealLog;
        }).collect(Collectors.toList());

        preDealService.insert(storeList);
    }

    /**
     * 初始化部门
     *
     * @param taskId
     * @param wxEnterpriseId
     * @param addList
     */
    private void dealDepartmentInit(String taskId, String wxEnterpriseId, List<SyncSingleDealDTO> addList) {
        Set<String> addListMid = addList.stream().map(dto -> dto.getRelatedId()).collect(Collectors.toSet());
        dealDepartmentToMq(taskId, addListMid, SyncTaskStatusEnum.group_sync);
    }

    /**
     * 放入mq处理部门数据
     *
     * @param taskId
     * @param dealList
     */
    private void dealDepartmentToMq(String taskId, Set<String> dealList, SyncTaskStatusEnum syncTaskStatusEnum) {
        //预处理分组任务
        syncTaskService.updateTaskStatus(taskId, syncTaskStatusEnum.getVal());
        dealList.forEach(relationId -> {
            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
            dealParamMqDTO.setTaskId(taskId);
            dealParamMqDTO.setData(relationId);
            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
            GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
            try {
                clientInstance.sendMessage("departmentSyncDealMq", JSONObject.toJSONString(dealParamMqDTO));
            } catch (Exception e) {
                logger.info("发送失败:{},{}", taskId, relationId);
                e.printStackTrace();
            }
        });
    }

    @Override
    public List getListTasks(String params) {
        return null;
    }

    @Override
    public void dealSingle(Object obj) throws TaskAllocationException {

    }

    @Override
    public void computed(TaskAllocationComputed computed) {

    }

    @Override
    public Object singleClass() {
        return null;
    }
}
